local _, BPGP = ...

local GUI = BPGP.GetModule("GUI")

local L = LibStub("AceLocale-3.0"):GetLocale("BPGP")
local DLG = LibStub("LibDialog-1.0")

local Common = BPGP.GetLibrary("Common")

local DataManager = BPGP.GetModule("DataManager")
local StandingsManager = BPGP.GetModule("StandingsManager")
local ChecksManager = BPGP.GetModule("ChecksManager")

local private = {}

----------------------------------------
-- Public interface
----------------------------------------

function GUI.CreateBPGPToolsFrame()
	local f = GUI.CreateCustomFrame("BPGPToolsFrame", BPGPFrame, "DefaultBackdrop,DialogHeader,CloseButtonHeaderX")
  GUI.RegisterSideframe(f)

  f:Hide()
  f:SetPoint("TOPLEFT", BPGPFrame, "TOPRIGHT", -6, -27)
  f:SetWidth(162)
	f:SetHeight(310)
  
  f:SetHeader(L["Tools"])
  
  local massLabel = f:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
  massLabel:SetText(L["Mass operations"])
  massLabel:SetPoint("TOP", f, 0, -20)
  massLabel:SetPoint("LEFT", f, "LEFT", 18, 0)

  local award = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
  award:SetNormalFontObject("GameFontNormalSmall")
  award:SetHighlightFontObject("GameFontHighlightSmall")
  award:SetDisabledFontObject("GameFontDisableSmall")
  award:SetHeight(GUI.GetButtonHeight())
  award:SetPoint("LEFT", massLabel, "LEFT", -2, 0)
  award:SetPoint("TOP", massLabel, "BOTTOM", 0, -5)
  award:SetText(L["Mass BP Award"])
  award:SetWidth(130)
  award:SetScript("OnClick", function() GUI.ToggleSideFrame(BPGPSideFrame2) end)
  award:SetScript("OnUpdate", function(self)
    if DataManager.IsAllowedDataWrite() then
      self:Enable()
    else
      self:Disable()
    end
  end)
  GUI.RegisterRaidOlnyControl(award)

  local decay = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
  decay:SetNormalFontObject("GameFontNormalSmall")
  decay:SetHighlightFontObject("GameFontHighlightSmall")
  decay:SetDisabledFontObject("GameFontDisableSmall")
  decay:SetHeight(GUI.GetButtonHeight())
  decay:SetPoint("TOP", award, "BOTTOM", 0, -5)
  decay:SetText(L["Decay"])
  decay:SetWidth(130)
  decay:SetScript("OnClick", function() DLG:Spawn("BPGP_DECAY_BPGP", DataManager.GetDecayPercent()) end)
  decay:SetScript("OnUpdate", function(self)
    if DataManager.IsAllowedDataWrite() and DataManager.GetDecayPercent() > 0 then
      self:Enable()
    else
      self:Disable()
    end
  end)
  
  local serviceLabel = f:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
  serviceLabel:SetText(L["Utilities"])
  serviceLabel:SetPoint("LEFT", f, "LEFT", 18, 0)
  serviceLabel:SetPoint("TOP", decay, "BOTTOM", 0, -10)
  
  local lootCheckButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
  lootCheckButton:SetNormalFontObject("GameFontNormalSmall")
  lootCheckButton:SetHighlightFontObject("GameFontHighlightSmall")
  lootCheckButton:SetDisabledFontObject("GameFontDisableSmall")
  lootCheckButton:SetHeight(GUI.GetButtonHeight())
  lootCheckButton:SetText(L["Whose Loot Check"])
  lootCheckButton:SetWidth(130)
  lootCheckButton:SetPoint("LEFT", serviceLabel, "LEFT", -2, 0)
  lootCheckButton:SetPoint("TOP", serviceLabel, "BOTTOM", 0, -5)
  lootCheckButton:SetScript("OnClick", function(self)
    ChecksManager.RequestLootCheck()
  end)
  
  local versionCheckButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
  versionCheckButton:SetNormalFontObject("GameFontNormalSmall")
  versionCheckButton:SetHighlightFontObject("GameFontHighlightSmall")
  versionCheckButton:SetDisabledFontObject("GameFontDisableSmall")
  versionCheckButton:SetHeight(GUI.GetButtonHeight())
  versionCheckButton:SetText(L["Versions Check"])
  versionCheckButton:SetWidth(130)
  versionCheckButton:SetPoint("TOP", lootCheckButton, "BOTTOM", 0, -5)
  versionCheckButton:SetScript("OnClick", function(self)
    ChecksManager.RequestVersionCheck()
  end)
  
  local toggleCooldownButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
  toggleCooldownButton:SetNormalFontObject("GameFontNormalSmall")
  toggleCooldownButton:SetHighlightFontObject("GameFontHighlightSmall")
  toggleCooldownButton:SetDisabledFontObject("GameFontDisableSmall")
  toggleCooldownButton:SetHeight(GUI.GetButtonHeight())
  toggleCooldownButton:SetText(L["Raid Cooldowns"])
  toggleCooldownButton:SetWidth(130)
  toggleCooldownButton:SetPoint("TOP", versionCheckButton, "BOTTOM", 0, -5)
  toggleCooldownButton:SetScript("OnClick", function(self)
    GUI.ToggleBPGPCooldownsFrame()
  end)
  
  local toggleWipeProtectionButton = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
  toggleWipeProtectionButton:SetNormalFontObject("GameFontNormalSmall")
  toggleWipeProtectionButton:SetHighlightFontObject("GameFontHighlightSmall")
  toggleWipeProtectionButton:SetDisabledFontObject("GameFontDisableSmall")
  toggleWipeProtectionButton:SetHeight(GUI.GetButtonHeight())
  toggleWipeProtectionButton:SetText(L["Wipe Protection"])
  toggleWipeProtectionButton:SetWidth(130)
  toggleWipeProtectionButton:SetPoint("TOP", toggleCooldownButton, "BOTTOM", 0, -5)
  toggleWipeProtectionButton:SetScript("OnClick", function(self)
    GUI.ToggleBPGPWipeProtectionFrame()
  end)
  
end