local _, BPGP = ...

local MasterLooter = BPGP.NewModule("MasterLooter")

local L = LibStub("AceLocale-3.0"):GetLocale("BPGP")
local DLG = LibStub("LibDialog-1.0")

local Debugger = BPGP.Debugger
local Common = BPGP.GetLibrary("Common")
local Coroutine = BPGP.GetLibrary("Coroutine")

local DataManager = BPGP.GetModule("DataManager")
local AnnounceSender = BPGP.GetModule("AnnounceSender")

local private = {
  dbDefaults = {
    profile = {
      enabled = true,
      rollAnnounceMedium = "RAID_WARNING",
      collectRollResults = false,
      resetRollResults = false,
      bossPopup = false,
      lootAnnounce = true,
      lootAnnounceThreshold = 4,
      lootAutoAdd = false,
      lootAutoAddThreshold = 4, -- 0 - Poor,  1 - Common, 2 - Uncommon, 3 - Rare, 4 - Epic, 5 - Legendary
  --    bossWipePopup = false,
      lootPopup = false,
      lootPopupThreshold = 4, -- 0 - Poor,  1 - Common, 2 - Uncommon, 3 - Rare, 4 - Epic, 5 - Legendary
      standbySupport = false,
      standbyRequests = false,
      standbyAnnounceMedium = "GUILD",
      announceMedium = "GUILD",
      announceEvents = {
        ["BankedItem"] = true,
        ["FreeItem"] = true,
        ["MemberAwardedBP"] = true,
        ["MemberAwardedGP"] = true,
        ["RaidAwardedBP"] = true,
        ["RaidAwardedGP"] = true,
      },
    },
  },
  ignoredItemIds = {
    [20725] = true, -- Nexus Crystal
  },
  trackersEnabled = false,
}
Debugger.Attach("MasterLooter", private)

function MasterLooter:OnAddonLoad()
  self.db = BPGP.db:RegisterNamespace("MasterLooter", private.dbDefaults)
end

function MasterLooter:OnEnable()
end

function MasterLooter:OnDisable()
end

function MasterLooter.SetupTrackers()
  if DataManager.SelfInRaid() then
    Debugger("Looking for not loaded common trackers to load...")
    BPGP.GetModule("LootTracker"):Enable()
    BPGP.GetModule("RollTracker"):Enable()
    BPGP.GetModule("DistributionTracker"):Enable()
    if DataManager.SelfML() then
      Debugger("Looking for not loaded ML-only trackers to load...")
      BPGP.GetModule("BossTracker"):Enable()
      BPGP.GetModule("StandbyTracker"):Enable()
    else
      BPGP.GetModule("BossTracker"):Disable()
      BPGP.GetModule("StandbyTracker"):Disable()
    end
  else
    Debugger("Looking for loaded trackers to unload...")
    BPGP.GetModule("BossTracker"):Disable()
    BPGP.GetModule("LootTracker"):Disable()
    BPGP.GetModule("RollTracker"):Disable()
    BPGP.GetModule("DistributionTracker"):Disable()
    BPGP.GetModule("StandbyTracker"):Disable()
  end
end

----------------------------------------
-- Public interface
----------------------------------------
  
function MasterLooter:LootItemsAnnounce(itemLinks)
  local err = nil
  if not DataManager.SelfInRaid() then
    err = L["you should be in a raid group"]
  elseif not DataManager.SelfML() then
    err = L["you should be a Master Looter"]
  elseif #itemLinks == 0 then
    err = L["please add some items first"]
  end
  if err then
    BPGP.Print(L["Failed to announce the loot list: %s!"]:format(err))
    return
  end
  AnnounceSender.AnnounceTo(MasterLooter.db.profile.rollAnnounceMedium, L["Loot list: "] .. table.concat(itemLinks, " "))
end

function MasterLooter.StartDistribution(mode, itemLink, comment)
  local err = nil
  if not DataManager.SelfInRaid() then
    err = L["you should be in a raid group"]
  elseif not DataManager.SelfML() then
    err = L["you should be a Master Looter"]
  end
  if err then
    BPGP.Print(L["Failed to announce new distribution: %s!"]:format(err))
    return
  end
  local medium = MasterLooter.db.profile.rollAnnounceMedium
  if mode == 1 then
    AnnounceSender.AnnounceTo(medium, L["Starting /roll"] .. " " .. itemLink .. " " .. comment)
  elseif mode == 2 or mode == 3 then
    AnnounceSender.AnnounceTo(medium, L["Starting distribution"] .. " " .. itemLink .. " " .. comment) 
  end
  BPGP.GetModule("DistributionTracker").StartDistribution(mode, itemLink, comment)
end

function MasterLooter.ShowLootPopup(playerName, itemId, itemLink, itemQuantity)
  if not DataManager.SelfML() or not DataManager.IsAllowedDataWrite() or private.ignoredItemIds[itemId] then return end
  BPGP.GetModule("DistributionTracker").StopDistribution(itemLink)
  Coroutine:RunAsync(private.AsyncShowLootPopup, playerName, itemId, itemLink, itemQuantity)
  BPGP.GetModule("GUI").LootItemsRemove(itemLink)
end

function private.AsyncShowLootPopup(playerName, itemId, itemLink, itemQuantity)
  while DLG:ActiveDialog("BPGP_CONFIRM_GP_CREDIT") do
    Coroutine:Sleep(0.1)
  end
  DLG:Spawn("BPGP_CONFIRM_GP_CREDIT", {name = playerName, item = itemLink, icon = GetItemIcon(itemId)})
end

----------------------------------------
-- Module options
----------------------------------------

